# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Vår (NDLA)
# Oppgave 4 c) Aritmetisk- og geometrisk rekke - Uker før tilbud 2 til sammen gir mer lønn enn tilbud 1 

an = 100            # setter startverdi
bn = 100            # setter startverdi
san = an
sbn = bn
n=1
while sbn <= san:
    n = n + 1 # oppdaterer verdien til n
    an = an + 10      # oppdaterer verdien til an
    bn = bn*1.05      # oppdaterer verdien til bn
    san = san + an    # oppdaterer verdien til san
    sbn = sbn + bn    # oppdaterer verdien til sbn

print(f"Etter {n} uker vil hun for første gang ha fått mer "
"til sammen med tilbud 2 enn med tilbud 1.")



# 🚀 programmering.no | 🤓 matematikk.as
# - Regn ut summen av elementene i en liste med for-løkke

# Input
liste = input("Listen som summeres: ") # Husk [ ], f.eks. [1, 2, 3, 4]

# Enkel funksjon som klipper opp inp-str til liste med int-el
def str_til_int_liste(str_liste = list()):

    int_liste = list()
    el = str()
    for i in range(len(str_liste)):
        if str_liste[i] == " ": pass
        elif str_liste[i] == "[": pass
        elif str_liste[i] == "]":
            int_liste.append(int(el))
            el = ""
        elif str_liste[i] == ",":
            int_liste.append(int(el))
            el = ""
        else: el += str_liste[i]
    return int_liste

# Funksjon som summerer el i en liste
def sum_liste(liste):

	# Nullstill
	sum = 0

	# Oppdater sum for hver iterasjon
	for el in liste:
		sum += el

	return(sum)

# Input-string endres til liste med int-el
liste = str_til_int_liste(liste)

# Sum
sum = sum_liste(liste)

# Print
print(f"Liste                : {liste}")
print(f"Summen av elementene : {sum}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Add (legg til) element til sett

# Variabler
sett = {"bil", "tog", "sykkel"}

# Print originalt sett
print(f"Originalt sett: {sett}")

# Add et element til settet
sett.add("fly")

# Print oppdatert sett
print(f"Oppdatert sett: {sett}")



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Matematikk AS)
# Oppgave 3 b) Sparekonto - Påstand: Dobbelt så lang tid 
# - Løser oppgaven med CAS i Python
# - Prosentvis vekst er en eksponentialfunksjon som kan skrives på formen
#   - f(x) = a * b^x, f(0) = a (startverdi), b > 0 (vekstfaktor)

from sympy import Eq, Reals, solveset, Symbol

# Konstanter og CAS-variabler (symbol)
B_P                = Symbol("B_P")     # Beløp etter n år (Per)
B_K                = Symbol("B_K")     # Beløp etter n år (Kåre)
B_30000            = 30000             # 30 000,- spart
I                  = Symbol("I")       # Innskuddet (startbeløpet inn på konto)
V_P                = Symbol("V_P")     # Vekstfaktor (Per)
V_K                = Symbol("V_K")     # Vekstfaktor (Kåre)
p_rente_p          = 3                 # Årlig rente i banken (Per)
p_rente_k          = 6                 # Årlig rente i banken (Per)
n                  = Symbol("n")       # Antall år med sparing
n_8                = 8                 # Periode på 8 år
n_dobbelt_p        = float()           # År med sparing (Per)
n_dobbelt_k        = float()           # År med sparing (Kåre)
n_forhold          = float()           # Forholdet n_dobbelt_p / n_dobbelt_k
_blokk             = 1                 # 0: Av, 1: På, Skjul/debug deler av koden med if-blokker

# Definerer funksjoner
def vekstfaktor(fortegn = str(),
                p       = float(),
                desimal = -1,
                rund    = -1):
    
    # Vekstfaktor er definert som V = 1 ± p / 100, p: prosentvis vekst [%]
    v = 0.0
    if fortegn == "+": v = 1 + p / 100 # "+": Øker
    if fortegn == "-": v = 1 - p / 100 # "-": Minker
    
    # Runder av svaret
    if rund != -1: v = round(v, rund)
    
    return v

def los_sett(variabel = Symbol(""),
             vs       = Symbol(""),
             hs       = Symbol(""),
             rund     = None):

    # Lager likningen med Eq(vs, hs) 
    likning = Eq(vs, hs)

    # Løser likningen for aktuell variabel med solveset() og får løsnings-settet på formen {1234.56789}
    losning_set = solveset(likning, variabel, domain = Reals) # Løsnings-domene i ℝ (Reals)

    # Henter elementet i fra løsnings-settet, f.eks. 1234.56789
    losning = losning_set.args[0] # [0] er første element (det eneste her)

    # Runder av f.eks. 1234.56789 -> 1234.6
    losning = round(losning, rund)

    return losning

# Definerer uttrykket for vekstfaktoren, v = 1 + p / 100
V_P = vekstfaktor("+", p_rente_p)
V_K = vekstfaktor("+", p_rente_k)

# a) Finn innskuddet til Per
if _blokk == 1:

    # Definerer uttrykket for B
    B = I * V_P**n # B = I * 1.03**n (en ukjente, I og n)

    # Setter n = 8 inn i B og definerer det nye uttrykket som B_8 
    B_8 = B.subs(n, n_8) # B_8 = I * 1.03**8

    # Løser likningen 30000 = I * 1.03**8 for I
    I = los_sett(variabel = I,
                 vs       = B_30000,
                 hs       = B_8,
                 rund     = None) # I = 23682

    # Svar-setninger
    svar_a_liste = list()
    svar_a_liste.append(f"")
    svar_a_liste.append(f"Oppg a)")
    svar_a_liste.append(f"")
    svar_a_liste.append(f"- Dersom Per skal ha kr {B_30000} på kontoen sin etter {n_8} år,")
    svar_a_liste.append(f"  så må han ha et innskudd på ca. kr {I}")

    # Print svar-setninger
    for svar in svar_a_liste: print(svar)

# b) Påstand: Dobbelt så lang tid
if _blokk == 1:

    # Definerer uttrykkene for B_P og B_K
    B_P = I * V_P**n
    B_K = I * V_K**n

    # Løser likningen 2 * 23682 = 23682 * 1.03**n for n
    n_dobbelt_p = los_sett(variabel = n,
                           vs       = 2 * I,
                           hs       = B_P,
                           rund     = 2)

    # Løser likningen 2 * 23682 = 23682 * 1.06**n for n
    n_dobbelt_k = los_sett(variabel = n,
                           vs       = 2 * I,
                           hs       = B_K,
                           rund     = 2)

    # Regner ut forholdet n = 23.45 / 11.90 = 1.97 
    n_forhold = n_dobbelt_p / n_dobbelt_k

    # Runder av svaret
    n_forhold = round(n_forhold, 2)

    # Svar-setninger
    svar_b_liste = list()
    svar_b_liste.append(f"")
    svar_b_liste.append(f"Oppg b)")
    svar_b_liste.append(f"")
    svar_b_liste.append(f"- Nei, det vil ikke gå nøyaktig dobbelt så lang tid, men ca.")
    svar_b_liste.append(f"  {n_dobbelt_p} år / {n_dobbelt_k} år = {n_forhold} ganger så lang tid")
    svar_b_liste.append(f"- Vi kan også vise at likningen ikke kan løses")
    svar_b_liste.append(f"     2 * B_P = 2 * B_K | 1 / 2")
    svar_b_liste.append(f"     B_P = B_K")
    svar_b_liste.append(f"     I * 1.03^(2n) = I * 1.06^n | 1 / I")
    svar_b_liste.append(f"     1.03^(2n) = 1.06^n")
    svar_b_liste.append(f"     (1.03^2)^n = 1.06^n, a^(nm) = (a^n)^m")
    svar_b_liste.append(f"     1.0609^n ≠ 1.06^n")
    svar_b_liste.append(f"")
    svar_b_liste.append(f"- Påstanden er derfor ikke riktig")

    # Print svar-setninger
    for svar in svar_b_liste: print(svar)



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Høst (Matematikk AS)
# Oppgave 1 d) Etterspørsel - Størst overskudd (optimal produksjonsmengde) 
# - Løser oppgaven med CAS i Python
# - Kopier denne kommandoen i terminalen for å importere sympy: 
#   $ pip install sympy

import matematikk as mt

# Variabler
x                   = mt.Symbol("x")        # Etterspørsel (antall produserte enheter)
p                   = mt.Symbol("p")        # Pris per enhet
I                   = mt.Symbol("I")        # Inntekt
K                   = mt.Symbol("K")        # Kostnad
svar_d_ls           = list()                # Svar-setninger
desimal             = None                  # Avrunding

# Definerer uttrykket for pris-funksjonen, p(x)
p = 79 - 12.2 * mt.ln(x)

# Definerer uttrykket for inntekts-funksjonen, I(x)
I = x * p

# Deriverer I mhp. x og får dI = 66.8 - 12.2*log(x)
dI = mt.deriver(I, x) # I'(x)

# Definerer uttrykket for kostnads-funksjonen, K(x)
K = 0.021 * x**2 + 10 * x + 910

# Deriverer K mhp. x og får dK = 0.042*x + 10
dK = mt.deriver(K, x) # K'(x)

# Løser likningen dI = dK mhp. x
x_opt = mt.superløs(variabel = x,
                    vs       = dI,
                    hs       = dK,
                    rund     = desimal,
                    debug    = -1)

# Svar-setninger
svar_d_ls.append(f"")
svar_d_ls.append(f"Oppg d)")
svar_d_ls.append(f"")
svar_d_ls.append(f"- Grense-inntekten er lik grense-kostnaden ved {x_opt} enheter")
svar_d_ls.append(f"- Dette betyr at {x_opt} enheter gir størst overskudd fordi:")
svar_d_ls.append(f"     O`(x) = 0, max overskudd")
svar_d_ls.append(f"     O`(x) = I`(x) - K`(x), definisjonen av overskudd")
svar_d_ls.append(f"     0 = I`(x) - K`(x)")
svar_d_ls.append(f"     K`(x) = I`(x)")

# Print svar-setninger
for svar in svar_d_ls: print(svar)



# 🚀 programmering.no | 🤓 matematikk.as

from matematikk import superløs, diff, Symbol

def ekstremalpunkt_max(variabel = Symbol(""),
                       uttrykk  = Symbol(""),
                       rund     = -1,
                       debug    = -1):

    # Deriverer uttrykk mhp. variabel og får df
    df = diff(uttrykk, variabel) # f'(x)

    # Løser likningen df = 0 
    variabel_max = superløs(variabel = variabel,
                            vs       = df,
                            hs       = 0,
                            rund     = rund,
                            debug    = debug)

    return variabel_max


# Alias > 1
ekstremalpunkt_maks             = ekstremalpunkt_max
ekstremalpunkt_maksimalt        = ekstremalpunkt_max
toppunkt                        = ekstremalpunkt_max

# Alias > 2 > ...



# 🚀 programmering.no | 🤓 matematikk.as
# - Regn ut arealet av en sirkel

import math

# Verdier
r = 3

# Areal-funksjon for sirkel
def sirkel_areal_math(r):

    # Formel for arealet av sirkel
    areal = math.pi * pow(float(r), 2)

    return areal

# Areal
areal = sirkel_areal_math(r)

# Print
print(f"Arealet er:: {areal}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Se om listen er sortert i stigende eller synkende rekkefølge (monotonisk)
# - Kort versjon med all()

# Verdier
liste = [1, 2, 3, 4, 5, 6, 7]

# Funksjon som ser om listen er monotonisk eller ikke
def er_liste_monotonisk(liste):

	# Returner True hvis listen er monotonisk og False hvis ikke
	return (all(liste[i] <= liste[i + 1] for i in range(len(liste) - 1)) or
			all(liste[i] >= liste[i + 1] for i in range(len(liste) - 1)))

# Monotonisk eller ikke
er_monotonisk = er_liste_monotonisk(liste)

# Liste
print(f"Liste: {liste}")
if er_monotonisk == True:
	print(f"Er listen sortert? Ja")
if er_monotonisk == False:
	print(f"Er listen sortert? Nei")



# 🚀 programmering.no | 🤓 matematikk.as

import matematikk as mt

def tabell_print_effekt(n_perioder = 10, mat_fx = []):

    # Effekt
    _typ = str()
    if mat_fx != []: _typ = mat_fx[0]   # Hvilken effekt (foreløpig kun "matrix")
    if mat_fx == []: _typ = "matrix"    # Default

    # Matrix effekt
    if _typ == "matrix":

        # Perioder
        _n_perioder = n_perioder

        # Resett mat
        _mat_fx = []

        # Default settings
        if mat_fx == []:
                _mat_fx.append([                # Liste med japanske symbol (jp-sym) som hentes frem tilfeldig (alleer dobbelt så lange som vanlige char)
                    "グ", "ケ", "ゲ", "コ", "ゴ", "サ", "ザ", "シ", "ジ", "ス", "ズ", "セ", "ゼ", "ソ", "ゾ", "タ"
                ])
                _mat_fx.append(["0", "1"])      # Liste med 0 og 1 som blandes med jp-sym
                _mat_fx.append([                # Filter-parametre som gir en tilfeldig, dynamisk blanding mellom to lister (se nedenfor): 3 i dette scopet og 4 som argumenter inn i tabell
                    1.00, 0.10, 0.60, 0.80, 0.99, 0.10, 0.99
                ])
                _mat_fx.append(700000 / 10)     # Gjør at flere el kommer etter hverandre (startet med 692 852 som er ca. 700 000 / 140 = 5000 el etter hverandre)
                _mat_fx.append(692852)

        # Egne argumenter: Kun alle 4 elementene eller ingen (default)
        if mat_fx != []:
                _mat_fx.append(mat_fx[1])
                _mat_fx.append(mat_fx[2])
                _mat_fx.append(mat_fx[3])
                _mat_fx.append(mat_fx[4])

        # Argumenter
        matrix_jp_ls        = _mat_fx[0]
        matrix_null_en_ls   = _mat_fx[1]
        matrix_f_ls         = _mat_fx[2]
        matrix_cut_off      = _mat_fx[3]

        # Nullstill matrix-lister
        _matrix_jp_mix_ls   = []
        _matrix_null_ls     = []
        _matrix_jp_null_ls  = []

        # Filter > Sekvens med like js-sym etter hverandre blandet med tilfeldige
        j = 0
        k = 1 # Enten 1 (i intervall med like etter hverandre)
        m = 0
        for i in range(_n_perioder):

            # Sekvens med f.eks. 5000 jp-sym etter hverandre
            if k == 1:
                _r                                      = mt.random.randint(len(matrix_jp_ls)) # Sekvensen blandes med noen tilfeldige jp-sym
                if mt.random.random() > matrix_f_ls[0]:    _matrix_jp_mix_ls.append(f"{matrix_jp_ls[m]}") # m plukker samme jp-sym for 2 sekvenser
                else:                                   _matrix_jp_mix_ls.append(f"{matrix_jp_ls[_r]}")

            # Sekvens med f.eks. 5000 white space
            if k == 2:                                  _matrix_jp_mix_ls.append("  ") # Dobbel-space pga. jp-sym tar 2 plasser

            if j > 0    and j <= matrix_cut_off:                k = 1 # k er 1 så lenge j er i den 1. sekvensen
            if j > matrix_cut_off and j < matrix_cut_off * 2:   k = 2 # k blir 2 når j går over i den 2. sekvensen
            if j == matrix_cut_off * 2:
                j                                       = 0 # j nullstilles når den når 3. sekvens
                m                                       = mt.random.randint(len(matrix_jp_ls)) # m blir en ny tilfeldig index for et nytt jp-sym
            j                                           += 1    # j øker alltid med 1 og nullstilles for hver 3. sekvens

        # Filter > 0 blandet med 1
        for i in range(_n_perioder):
            if mt.random.random() > matrix_f_ls[1]:    _matrix_null_ls.append(f" {matrix_null_en_ls[0]}") # Dobbel-space pga. jp-sym tar 2 plasser
            else:                                   _matrix_null_ls.append(f" {matrix_null_en_ls[1]}") # -"-

        # Filter > jp blandet med 0 og 1
        for i in range(_n_perioder):
            if mt.random.random() > matrix_f_ls[2]:    _matrix_jp_null_ls.append(_matrix_jp_mix_ls[i])
            else:                                   _matrix_jp_null_ls.append(_matrix_null_ls[i])

        # Videre som argument inn i tabell
        _mat_matrix = ["matrix", _matrix_jp_null_ls, [0], matrix_f_ls] # (*) Trenger kun ett [0] som øker med belop_start_ink_he

    return _mat_matrix

def tabell_print_farge(txt = str(), tab_farge_typ = str()):

    # Farge
    if tab_farge_typ == "blå":          return f"{mt.Fore.BLUE}      {txt}{mt.Style.RESET_ALL}"
    if tab_farge_typ == "grønn":        return f"{mt.Fore.GREEN}     {txt}{mt.Style.RESET_ALL}"
    if tab_farge_typ == "magentea":     return f"{mt.Fore.MAGENTA}   {txt}{mt.Style.RESET_ALL}"
    if tab_farge_typ == "rød":          return f"{mt.Fore.RED}       {txt}{mt.Style.RESET_ALL}"
    if tab_farge_typ == "gul":          return f"{mt.Fore.YELLOW}    {txt}{mt.Style.RESET_ALL}"

    # Ingen farge (bypass) 
    return txt

def tabell_print_kolonne_padding(txt = str(), tab_bredde = int(), tab_fx = []):

    # Effekt
    if tab_fx != []:

        # Matrix-effekt: Foreløpig ingen advarsler default hvis ukjent effekt
        if tab_fx[0] == "matrix":

            # Padder celle-breddene i tabellen med white space (lik bredde)
            _txt                                        = str(txt)
            _bredde_diff                                = tab_bredde - len(str(_txt))
            for _ in range(_bredde_diff):               _txt += " "

            # Varierer posisjonen hvor matrix-sym skrives mellom 9 og 10
            _posisjon = int()
            if mt.random.random() > tab_fx[3][3]:      _posisjon = 10
            else:                                       _posisjon = 9

            # "Pulserende treshhold"-verdi som demper og forsterker (mer statisk uten)
            if mt.random.random() > tab_fx[3][4]:      _tresh = tab_fx[3][5]
            else:                                       _tresh = tab_fx[3][6]

            if mt.random.random() > _tresh: # _tresh:
                _ran_1 = mt.random.randint(4, 6)           # (4, 7) er mer bredde-variasjon for matrix-sym, men blir mer intens
                if _bredde_diff > _posisjon:
                    _txt = _txt[:_ran_1] + str(tab_fx[1][0]) + _txt[_ran_1 + 2:]
                _k      = tab_fx[2][0]              # (*) Det økende elementet fra while-løkken
                _el     = tab_fx[1][_k]             # Henter frem neste matrix-sym i matrisen
                _txt    = _txt.replace("      ", f"    {_el}", 1) # Bytter ut tomrom med matrix-sym

            return _txt

    # Ingen effekt
    if tab_fx == []:

        # Padder celle-breddene i tabellen med white space  (lik bredde)
        _txt    = str(txt)
        _rang   = tab_bredde - len(str(_txt))
        for _ in range(_rang): _txt += " "

        return _txt

def tabell_print_rad(tab_mat = [], i = int(), tab_bredde = int(), tab_is_farge = -1, tab_farge_typ = str(), tab_fx = []):

    # Rader (i) og kolonner (j)
    _kolonne = ""
    for j in range(len(tab_mat)):
        _kolonne += "|" + " " + "" + tabell_print_kolonne_padding(txt = tab_mat[j][i], tab_bredde = tab_bredde, tab_fx = tab_fx)

    # Farge
    if tab_is_farge == 1: _kolonne = tabell_print_farge(txt = _kolonne, tab_farge_typ = tab_farge_typ)

    # Print
    print(_kolonne)

def tabell_print(
        tab_mat                 = [[], []],
        tab_print_1_start       = 0,
        tab_print_1_slutt       = 0,
        tab_print_2_start       = 0,
        tab_print_2_slutt       = 0,
        tab_bredde              = int(),
        tab_is_farge            = -1,
        tab_farge_typ           = str(),
        tab_fx              = [],
        n_periode               = int()
    ):

    # Devx-setting for rask debug
    _devx_alle_rader = 0

    # DEV > Hardkodet variabel-navn > Variabler
    _n_periode = mt.sjekk_datatype(vari_navn = "n_periode", vari = n_periode, typ = int)

    # Print header
    tabell_print_rad(tab_mat = tab_mat, i = 0, tab_bredde = tab_bredde, tab_is_farge = tab_is_farge, tab_farge_typ = tab_farge_typ, tab_fx = [])

    # Print rader
    for i in range(1, _n_periode + 1):

        # Devx > Utvalgte rader
        if _devx_alle_rader == 0:

            # Print tabell-rad (begrenset rader) > Matrix
            if i >= tab_print_1_start and i <= tab_print_1_slutt:
                tabell_print_rad(tab_mat = tab_mat, i = i, tab_bredde = tab_bredde, tab_is_farge = tab_is_farge, tab_farge_typ = tab_farge_typ, tab_fx = tab_fx)
            if i >= tab_print_2_start and i <= tab_print_2_slutt:
                tabell_print_rad(tab_mat = tab_mat, i = i, tab_bredde = tab_bredde, tab_is_farge = tab_is_farge, tab_farge_typ = tab_farge_typ)

        # Devx > Alle rader
        if _devx_alle_rader == 1: tabell_print_rad(tab_mat = tab_mat, i = i, tab_bredde = tab_bredde, tab_is_farge = tab_is_farge, tab_farge_typ = tab_farge_typ)

def tabell_finn_el(tab_mat, i_pos, j_pos_header):

    # Sjekk datatype
    _i_pos = mt.sjekk_datatype(vari_navn = "i_pos", vari = i_pos, typ = int)

    # Finn elementet
    for j in range(len(tab_mat)):
        if tab_mat[j][0] == j_pos_header:
            return tab_mat[j][_i_pos]

    # Hvis ikke finner elementet
    return None

def tabell_økonomi_lag(
        tab_bredde          = int(),
        tab_mat             = [[], []],
        tab_desimal         = -1,
        tab_print           = 1,
        tab_print_1_start   = -1,
        tab_print_1_slutt   = -1,
        tab_print_2_start   = -1,
        tab_print_2_slutt   = -1,
        tab_is_farge        = -1,
        tab_farge_typ       = -1,
        tab_fx          = [],
        vf                  = float(),
        n_periode           = int(),
        aar_start           = int(),
        innskudd            = float(),
        innskudd_desimal    = -1,
        innskudd_d          = float(),
        innskudd_d_desimal  = -1,
        b_start_desimal     = -1,
        b_slutt_desimal     = -1
    ):

    # Print
    if (tab_print               == 1 and
        tab_print_1_start       == -1 and
        tab_print_1_slutt       == -1 and
        tab_print_2_start       == -1 and
        tab_print_2_slutt       == -1):
            tab_print_1_start     = 0
            tab_print_1_slutt     = n_periode

    # Resetting
    _blokk_         = 1
    _tab_mat        = tab_mat
    _vf             = vf
    _n_periode      = mt.sjekk_datatype(vari_navn = "n_periode", vari = n_periode, typ = int)
    _aar            = mt.sjekk_datatype(vari_navn = "aar", vari = aar_start, typ = int)
    _innskudd       = innskudd
    _innskudd_d     = innskudd_d
    _b_start        = _innskudd

    # Format
    _debug_mat      = 0
    _tab_size       = 6
    _tab_f_mat      = []
    _tab_ret_mat    = []
    for i in range(_tab_size):
        _tab_f_mat.append([])
        _tab_ret_mat.append([])

    # Lager midlertidig formatert matrise av matrise med data
    def mat_ops(typ, mat_data, mat_format):

        # _tab_ret_mat må være øverst for siste format
        for j in range(len(mat_data)):
            if mat_data[j][0] == "Periode":
                _tab_ret_mat[j]     = mt.deepcopy(mat_format[0])
                mat_format[0]       = mt.deepcopy(mat_data[j])

            if mat_data[j][0] == "År":
                _tab_ret_mat[j]     = mt.deepcopy(mat_format[1])
                mat_format[1]       = mt.deepcopy(mat_data[j])

            if mat_data[j][0] == "Innskudd":
                _tab_ret_mat[j]     = mt.deepcopy(mat_format[2])
                mat_format[2]       = mt.deepcopy(mat_data[j])

            if mat_data[j][0] == "Innskudd d":
                _tab_ret_mat[j]     = mt.deepcopy(mat_format[3])
                mat_format[3]       = mt.deepcopy(mat_data[j])

            if mat_data[j][0] == "B (start)":
                _tab_ret_mat[j]     = mt.deepcopy(mat_format[4])
                mat_format[4]       = mt.deepcopy(mat_data[j])

            if mat_data[j][0] == "B (slutt)":
                _tab_ret_mat[j]     = mt.deepcopy(mat_format[5])
                mat_format[5]       = mt.deepcopy(mat_data[j])

        if typ == 1: return mat_format
        if typ == 2: return _tab_ret_mat

    _tab_f_mat = mat_ops(1, _tab_mat, _tab_f_mat)

    # Debug
    if _debug_mat == 1:
        print(""); print("debug > format - header ::")
        for j in range(len(_tab_f_mat)): print(_tab_f_mat[j])

    # For-løkke som regner ut tabell-verdiene
    for i in range(1, _n_periode + 1):

        # Regner ut slutt-beløpet med vekstfaktor (avrundet)
        _b_slutt = _b_start * _vf

        # Periode (n)
        if _blokk_ == 1:

            # Tabell > Legg verdier i matrisen
            _tab_f_mat[0].append(i)

        # År
        if _blokk_ == 1:

            _tab_f_mat[1].append(_aar)

        # Innskudd
        if _blokk_ == 1:

            # Avrunding
            if innskudd_desimal != -1: _innskudd = round(_innskudd, innskudd_desimal)

            # Legger verdien i matrisen
            _tab_f_mat[2].append(_innskudd)

        # Innskudd d
        if _blokk_ == 1:

            # Avrunding
            if innskudd_d_desimal != -1: _innskudd_d = round(_innskudd_d, innskudd_d_desimal)

            # Legger verdien i matrisen
            _tab_f_mat[3].append(_innskudd_d)

        # Beløp (start)
        if _blokk_ == 1:

            # Avrunding
            if b_start_desimal != -1: _b_start = round(_b_start, b_start_desimal)

            # Legger verdien i matrisen
            _tab_f_mat[4].append(_b_start)

        # Beløp (slutt)
        if _blokk_ == 1:

            # Avrunding
            if b_slutt_desimal != -1: _b_slutt = round(_b_slutt, b_slutt_desimal)

            # Legger verdien i matrisen
            _tab_f_mat[5].append(_b_slutt)

        # Inkrement variabler
        _aar            += 1
        _innskudd       += _innskudd_d
        _b_start        = _b_slutt + _innskudd

    # Avrunding > Hele tabellen
    if tab_desimal != -1:
        for j in range(len(_tab_f_mat)):
            for i in range(len(_tab_f_mat[j])):
                if i > 0: _tab_f_mat[j][i] = round(_tab_f_mat[j][i], tab_desimal)

    # Debug
    if _debug_mat == 1:
        print(""); print("debug > format ::")
        for j in range(len(_tab_f_mat)): print(_tab_f_mat[j])

    # Lag return-mat som innkommende data-mat
    _tab_ret_mat = mat_ops(2, _tab_mat, _tab_f_mat)

    # Fjern tomme kolonner
    for j in range(len(_tab_ret_mat)):
        if _tab_ret_mat[j] == []: _tab_ret_mat.pop(j)

    # Debug
    if _debug_mat == 1:
        print(""); print("debug > ret ::")
        for j in range(len(_tab_ret_mat)): print(_tab_ret_mat[j])

    # Print tabell
    if tab_print == 1:
        tabell_print(
            tab_mat                 = _tab_ret_mat,
            tab_print_1_start       = tab_print_1_start,
            tab_print_1_slutt       = tab_print_1_slutt,
            tab_print_2_start       = tab_print_2_start,
            tab_print_2_slutt       = tab_print_2_slutt,
            tab_bredde              = tab_bredde,
            tab_fx              = tab_fx,
            tab_is_farge            = tab_is_farge,
            tab_farge_typ           = tab_farge_typ,
            n_periode               = _n_periode,
        )

    return _tab_ret_mat


# Alias > 1
tabell_okonomi_lag = tabell_økonomi_lag

# Alias > 2 > ...




# 🚀 programmering.no | 🤓 matematikk.as
# - Se om tallet er et primtall
# - Algoritmen kan optimaliseres ved å bytte ut int(n / 2) med int(sqrt(n))

# Input
n = input("Se om tallet er primtall: ")

# Funksjon for å se om tallet er primtall
def er_primtall(n):

    # Type cast inp-str -> int
    n = int(n)

    # Alle primtall er større enn 1
    if n > 1:

        # Iterer fra 2 til n / 2 + 1 (medium tempo)
        for i in range(2, int(n / 2) + 1):

            # n er ikke et primtall hvis det er delelig med i
            if n % i == 0:
                print(f"Er {n} et primtall? Nei")
                break

        # Hvis for-løkken fullføres uten break
        else: print(f"Er {n} et primtall? Ja")

    # Hvis tallet ikke er større enn 1 er det ikke primtall
    else: print(f"Er {n} et primtall? Nei")

# Primtall eller ikke
er_primtall(n)



# 🚀 programmering.no | 🤓 matematikk.as
# - Del en liste opp i mindre lister med n elementer i hver liste

# Input
liste = input("Listen som skal deles opp: ") # Husk [ ] og '', f.eks. ['gull', 'sølv', 'bronse']
steg = input("Antall elementer per liste: ")

# Enkel funksjon som klipper opp inp-str til liste med str-el
def str_til_str_liste(str_liste = list()):

    str_ferdig_liste = list()
    el = str()
    for i in range(len(str_liste)):
        if str_liste[i] == " ": pass
        elif str_liste[i] == "[": pass
        elif str_liste[i] == "]":
            str_ferdig_liste.append(el)
            el = ""
        elif str_liste[i] == ",":
            str_ferdig_liste.append(el)
            el = ""
        elif str_liste[i] == '"': pass # Fjerner doble hermetegn ("")
        elif str_liste[i] == "'": pass # Fjerner doble hermetegn ('')
        else: el += str_liste[i]
    return str_ferdig_liste

# Input-string endres til liste med str-el
liste = str_til_str_liste(liste)

# Del opp listen i del-lister med n elementer i hver del-liste
def oppdeling_lister_for_loop(liste):

    # Nullstill
    start      = 0
    slutt      = len(liste)
    del_lister = []

    # Itererer gjennom listen
    for i in range(start, slutt, int(steg)):
        del_lister.append(liste[i : i + int(steg)])

    return del_lister

# Del-lister
del_lister = oppdeling_lister_for_loop(liste)

# Print
print(f"Del-lister (hele): {del_lister}")
print("Del-lister (en og en):")
for del_liste in del_lister:
	print(f"{del_liste}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Eksempel på enkle, doble og tripple hermetegn

# Variabler
hermetegn_enkle     = 'Enkle hermetegn'
hermetegn_dobbel    = "Dobble hermetegn"
hermetegn_nøstet    = 'Det stod "programmering 24/7" på døren 👾'
hermetegn_trippel   = """Denne linjen
            går over
            3 linjer"""

# Print
print(hermetegn_enkle)
print(hermetegn_dobbel)
print(hermetegn_nøstet)
print(hermetegn_trippel)



# 🚀 programmering.no | 🤓 matematikk.as
# - Supert navnesystem (cases)

# Variabler
litenStorStorStorOsv = "camelCase"
StorStorStorOsv      = "PascalCase"
liten_underscore_❤️     = "snake_case"
STOR_UNDERSCORE      = "SCREAMING_SNAKE_CASE"

# Print
print(liten_underscore_❤️)



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Torodd)
# Oppgave 4 c) Sannsynlighet med 5 terninger - Bestem den største verdien av k som er slik at P(X ≥ k) > 0,8 

import numpy as np
rng = np.random.default_rng()

def sum_større_eller_lik(k):
    N = 1000000
    terningar = rng.integers(1, 7, size = (5, N))
    fem_kast = terningar.sum(axis=0)
    gunstige = sum(fem_kast >= k)
    p = gunstige / N
    return p

k = 30

while sum_større_eller_lik(k) < 0.8:
    k = k - 1

print(k)



# 🚀 programmering.no | 🤓 matematikk.as
# - Finn det minste tallet i en liste med sort()

# Verdier
liste = input("Finn minimum i listen: ") # Husk [ ], f.eks. [1, 2, 3, 4]

# Enkel funksjon som klipper opp inp-str til liste med int-el
def str_til_int_liste(str_liste = list()):

    int_liste = list()
    el = str()
    for i in range(len(str_liste)):
        if str_liste[i] == " ": pass
        elif str_liste[i] == "[": pass
        elif str_liste[i] == "]":
            int_liste.append(int(el))
            el = ""
        elif str_liste[i] == ",":
            int_liste.append(int(el))
            el = ""
        else: el += str_liste[i]
    return int_liste

# Funksjon for å finne min el i en liste med sort()
def minimum_sort(liste):

	# Sorterer listen i synkende rekkefølge med reverse-argument i sort()
	liste.sort(reverse=True)

	# Min el er siste el i listen (index -1)
	min = liste[-1]

	return min

# Input-string endres til liste med int-el
liste = str_til_int_liste(liste)

# Minimum
min = minimum_sort(liste)

# Print
print(f"Liste             : {liste}")
print(f"Minimum i listen  : {min}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Eksempel på tuple

# Variabler
tuppel = ("Nord", "Sør", "Øst", "Vest")

# Print hele tuppelen
print(tuppel)

# Print det første elementet i tuppelen
print(tuppel[0])



# 🚀 programmering.no | 🤓 matematikk.as
# - Liten hack for å inkludere øvre grense i range

# Variabler
intervall = range(4, 10 + 1) # Øvre grense + 1

# Nå er øvre grense 10 inkludert
for tall in intervall:
    print(tall)



# 🚀 programmering.no | 🤓 matematikk.as
# - Finn alle fibonacci-tallene til og med posisjon n
# - Fibonacci-rekken genereres av formelen F(n) = F(n-2) + F(n-1)
# - De første fibonacci-tallene er 0, 1, 1, 2, 3, 5, 8, 13, 21, ..., n

# Verdier
fib_1 		= 0 		# Det første fibonacci-tallet
fib_2 		= 1 		# Det andre fibonacci-tallet
fib_liste 	= list() 	# Liste med fibonacci-tall
n           = 10        # Til og med tall på posisjon n

# Funksjon for å printe fibonacci-listen
def fibonacci_liste_print(liste):
    print(f"Fibonacci-liste: {liste}")

# Legg til og print fibonacci-listen
if int(n) > 0:
    fib_liste.append(fib_1)
    fibonacci_liste_print(fib_liste)
if int(n) > 1:
    fib_liste.append(fib_2)
    fibonacci_liste_print(fib_liste)

# Rekursiv fibonacci-funksjon
def fibonacci_n_liste(n):

	# Type cast inp-str -> int
	n = int(n)

	# Ikke definert
	if n < 1:
		print("n (posisjon) må være et heltall")
		return "Ikke definert"

	# Det første fibonacci-tallet
	elif n <= len(fib_liste):
		return fib_liste[n - 1]

	# Regn ut det n`te fibonacci-tallet rekursivt
	else:

		# Rekursiv formel for fibonacci-tall
		fib_n = fibonacci_n_liste(n - 1) + fibonacci_n_liste(n - 2)

		# Legg til og print fibonacci-listen
		fib_liste.append(fib_n)
		fibonacci_liste_print(fib_liste)

		return fib_n

# Fibonacci-liste
fibonacci_n_liste(n)



# 🚀 programmering.no | 🤓 matematikk.as
# - Del en liste opp i mindre lister med n elementer i hver liste

import numpy as np

# Verdier
steg  = 4
liste = [
                    "a", "b", "c", "d", "e", "f", "g", "h", "i", "j",
                    "k", "l", "m", "n", "o", "p", "q", "r", "s", "s",
                    "u", "v", "w", "x", "y", "z"
                ]

# Del opp listen i del-lister med n elementer i hver del-liste
def oppdeling_lister_numpy(liste):

    # Del opp listen med numpy
    del_lister = np.array_split(liste, steg)

    return del_lister

# Del-lister
del_lister = oppdeling_lister_numpy(liste)

# Print
print(f"Del-lister (hele): {del_lister}")
print("Del-lister (en og en):")
for del_liste in del_lister:
	print(f"{del_liste}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Finn produktet av alle elementene i en liste med numpy

import numpy

# Verdier
liste = [2, 3, 10]

# Finn produktet med numpy
produkt = numpy.prod(liste)

# Print
print(f"Liste                   : {liste}")
print(f"Produktet av elementene : {produkt}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Se ascii-symbolene til hver char i en string

# Verdier
string = "Apple"

# Funksjon som printer ascii-symbolene til hver char i stringen
def ascii_print(string):

    # Print
    print("--------------")
    print("Char <-> ASCII")
    print("--------------")

    # Loop gjennom chars i stringen
    for char in string:
        ascii = ord(char)
        print(f"{char} <-> {ascii}")

# Print ascii-symboler
ascii_print(string)



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Vår (Matematikk AS)
# Oppgave 1 a) Timelønnen til yrkesgruppe - Gjennomsnittlig årlige prosentvis vekst 
# - Løser oppgaven med CAS i Python
# - Prosentvis vekst er en eksponentialfunksjon som kan skrives på formen
#   - f(x) = a * b^x, f(0) = a (startverdi), b > 0 (vekstfaktor)

from sympy import ConditionSet, core, Eq, FiniteSet, Intersection, nsolve, Reals, solve, solveset, Symbol

# Konstanter og CAS-variabler (symbol)
_blokk             = 1                 # 0: Av, 1: På, Skjul/debug deler av koden med if-blokker
G                  = Symbol("G")       # Timelønn x år etter 2008
G_0                = Symbol("G_0")     # Timelønnen i 2008
G_liste            = [                 # Timelønn
    272.55,
    285.50,
    307.30,
    314.00,
    327.60,
    340.10]
V                  = Symbol("V")       # Vekstfaktor
x                  = Symbol("x")       # Antall år med lønnsvekst
x_liste            = [                 # Årstall
    2008,
    2010,
    2013,
    2015,
    2019,
    2022
]
p                  = Symbol("p")       # Gjennomsnittlig årlige prosentvis vekst

# Definerer funksjoner
def vekstfaktor_cas(fortegn = str(), p = Symbol("")):
    
    # Vekstfaktor er definert som V = 1 ± p / 100, p: prosentvis vekst [%]
    v = Symbol("v")
    if fortegn == "+": v = 1 + p / 100 # "+": Øker
    if fortegn == "-": v = 1 - p / 100 # "-": Minker
    
    return v

def superløs(variabel = Symbol(""),
             vs       = Symbol(""),
             hs       = Symbol(""),
             likning  = list(),
             rund     = -1,
             debug    = -1):

    vis_datatype = True
    losning_set, losning_set_sub = set(), set()
    losning_liste, losning_element, losning_rund_liste, losning_ut = list(), list(), list(), list()
    losning_rund_status, sett_typ, likn_ant, los_ant, _v_avrund_typ = str(), str(), str(), str(), str()
    losning_rund = float()

    if type(likning) != list: likning_tmp = likning; likning = []; likning.append(likning_tmp)

    if len(likning) == 0:
        likn_ant = "En likning"
        if type(variabel) == type(list()): variabel = variabel[0]
        if type(vs) != core.mul.Mul and type(vs) == type(list()): vs = vs[0]
        if type(hs) != core.mul.Mul and type(hs) == type(list()): hs = hs[0]
        _likning = Eq(vs, hs); losning_set = solveset(_likning, variabel, domain = Reals)

        if len(likning) == 0 and type(losning_set) == FiniteSet:
            sett_typ = "FiniteSet";
            if len(losning_set) == 1: los_ant = "en løsning"
            if len(losning_set) > 1: los_ant = str(len(losning_set)) + " " + "løsninger"

        if len(likning) == 0 and type(losning_set) == ConditionSet:
            sett_typ = "ConditionSet"; losning_element = nsolve(_likning, variabel, 1)
            los_ant = "en numerisk løsning"; losning_set = []; losning_set.append(losning_element)

        if len(likning) == 0 and type(losning_set) == Intersection:
            sett_typ = "Intersection"; losning_set_sub = losning_set.args[1]
            losning_element = losning_set_sub.args; losning_set = [];
            for el in losning_element: losning_set.append(el); rund = -1
            if len(losning_set) == 1: los_ant = "en løsning som er et uttrykk"
            if len(losning_set) > 1: los_ant = str(len(losning_set)) + " " + "løsninger som er uttrykk"

    if len(likning) > 0:
        sett_typ = ""; likn_ant = "Flere likninger (likningssett)"
        losning_set = solve(likning, variabel, dict=True)
        for i in range(len(losning_set)):
            koordinat_verdi_liste = list(); koordinat_variabel_rund_liste = list()
            for koordinat_variabel, koordinat_verdi in losning_set[i].items():
                koordinat_verdi_liste.append(koordinat_verdi)
                losning_rund = round(float(koordinat_verdi), rund)
                koordinat_variabel_rund_liste.append(losning_rund)
            losning_liste.append(koordinat_verdi_liste); losning_rund_liste.append(koordinat_variabel_rund_liste)

    for losning in losning_set:
        if rund == -1:
            _v_avrund_typ = "(eksakt)"; losning_rund_status = "Nei"; losning_rund_liste = ""
            if len(likning) == 0: losning_liste.append(losning); losning_ut = losning_liste
            if len(likning) > 0: losning_ut = losning_liste

        if rund != -1:
            _v_avrund_typ = "(avrundet)"; losning_rund_status = "Ja"
            if len(likning) == 0:
                losning_liste.append(losning); losning_rund = round(losning, rund)
                losning_rund_liste.append(losning_rund); losning_ut = losning_rund_liste
            if len(likning) > 0: losning_ut = losning_rund_liste

    if len(losning_ut) == 1: losning_ut = losning_ut[0]

    if debug == 1:
        print("")
        print(f"*** Debug ****")
        if vis_datatype == True:
            print(f"Data-type                        :: {sett_typ}")
        print(f"Løsnings-type                    :: {likn_ant} med {los_ant} {_v_avrund_typ}")
        print(f"Løsnings-sett                    :: {losning_set}")
        print(f"Løsnings-element(er)             :: {losning_liste}")
        print(f"Avrunding (ja/nei)               :: {losning_rund_status}")
        print(f"Løsnings-elemente(er) avrundet   :: {losning_rund_liste}")
        print(f"-----------------------------------")
        print(f"Løsning ut (returnert) -----------> {losning_ut}")
        print(f"-----------------------------------")

    return losning_ut

# a) Gjennomsnittlig årlige prosentvis vekst
if _blokk == 1:
 
    # Definerer uttrykket for vekstfaktoren, v = 1 + p / 100
    V = vekstfaktor_cas("+", p)

    # Definerer uttrykket for årlig gjennomsnittlig vekst, f(x) = a * b^x
    G = G_0 * V**x

    # Regner ut antall år for perioden 2008–2022 
    x_periode = x_liste[-1] - x_liste[0] # x_periode = 14

    # Setter x = x_periode inn i G og definerer det nye uttrykket som G_periode 
    G_periode = G.subs(x, x_periode) # G_periode = G_0 * V**14

    # Setter G_0 = G_liste[0] inn i G_periode og oppdaterer uttrykket
    G_periode = G_periode.subs(G_0, G_liste[0]) # G_periode = 272.55 * V**14

    # Regner ut p = 1.6 i V fra G_periode = 272.55 * V**14 
    p_periode = superløs(variabel = p,
                         vs       = G_periode, # 272.55 * V**14
                         hs       = G_liste[-1], # 340.10
                         rund     = 1,
                         debug    = -1)[1]

    svar_a_liste = list()
    svar_a_liste.append(f"")
    svar_a_liste.append(f"Oppg a)")
    svar_a_liste.append(f"")
    svar_a_liste.append(f"- I årene 2008-2022 var den gjennomsnittlige årlige")
    svar_a_liste.append(f"  prosentvise veksten ca. {p_periode} %")

    # Print svar-setninger
    for svar in svar_a_liste: print(svar)



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Vår (Matematikk AS)
# Oppgave 1 a) Timelønnen til yrkesgruppe - Gjennomsnittlig årlige prosentvis vekst 
# - Løser oppgaven med CAS i Python
# - Prosentvis vekst er en eksponentialfunksjon som kan skrives på formen
#   - f(x) = a * b^x, f(0) = a (startverdi), b > 0 (vekstfaktor)

from matematikk import Symbol, superløs, vekstfaktor_cas

# Konstanter og CAS-variabler (symbol)
G                  = Symbol("G")       # Timelønn x år etter 2008
G_0                = Symbol("G_0")     # Timelønnen i 2008
V                  = Symbol("V")       # Vekstfaktor
x                  = Symbol("x")       # Antall år med lønnsvekst
p                  = Symbol("p")       # Gjennomsnittlig årlige prosentvis vekst
_blokk             = 1                 # 0: Av, 1: På, Skjul/debug deler av koden med if-blokker

# Tabell
x_liste            = [2008,   2010,   2013,   2015,   2019,   2022]   # Årstall
G_liste            = [272.55, 285.50, 307.30, 314.00, 327.60, 340.10] # Timelønn

# a) Gjennomsnittlig årlige prosentvis vekst
if _blokk == 1:

    # Definerer uttrykket for vekstfaktoren, v = 1 + p / 100
    V = vekstfaktor_cas("+", p)

    # Definerer uttrykket for årlig gjennomsnittlig vekst, f(x) = a * b^x
    G = G_0 * V**x

    # Regner ut antall år for perioden 2008–2022 
    x_periode = x_liste[-1] - x_liste[0]

    # Setter x = x_periode inn i G og definerer det nye uttrykket som G_periode 
    G_periode = G.subs(x, x_periode)

    # Setter G_0 = G_liste[0] inn i G_periode og oppdaterer uttrykket
    G_periode = G_periode.subs(G_0, G_liste[0])

    # Regner ut p = 1.6 i V fra G_periode = 272.55 * V**14 
    p_periode = superløs(variabel = p,
                         vs       = G_periode, # 272.55 * V**14
                         hs       = G_liste[-1], # 340.10
                         rund     = 1,
                         debug    = -1)[1]

    svar_a_liste = list()
    svar_a_liste.append(f"")
    svar_a_liste.append(f"Oppg a)")
    svar_a_liste.append(f"")
    svar_a_liste.append(f"- I årene 2008-2022 var den gjennomsnittlige årlige")
    svar_a_liste.append(f"  prosentvise veksten ca. {p_periode} %")

    # Print svar-setninger
    for svar in svar_a_liste: print(svar)



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Høst (Matematikk AS)
# Oppgave 1 c) og d) Alle svar - eksamen - DEMO
# - Løser alle eksamens-oppgaver med CAS i Python (DEMO)
# - Kopier denne kommandoen i terminalen for å importere matematikk:
#   $ pip install matematikk

import matematikk as mt

alle_svar = mt.eksamen(
    p = 79 - 12.2 * mt.ln(mt.Symbol("x"))
) 




# 🚀 programmering.no | 🤓 matematikk.as

import matematikk as mt

def okonomi(variabel = mt.Symbol(""),
            uttrykk  = mt.Symbol(""),
            desimal  = None,
            debug    = -1):

    # overskudd_max() er en undergruppe av ekstremalpunkt_max()
    variabel_max = mt.ekstremalpunkt_max(variabel = variabel,
                                         uttrykk  = uttrykk,
                                         rund     = desimal,
                                         debug    = debug)

    return variabel_max

# Alias > 1 > ...

# Alias > 2 > ...




# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Matematikk AS)
# Oppgave 1 b) Sofa-produksjon til møbelfabrikk - Størst overskudd 
# - Løser oppgaven med CAS i Python
# - Kopier kommandoene i terminalen og importer pakkene 
#   $ pip install numpy
#   $ pip install sympy
#   $ pip install matematikk

from numpy import polyfit
from sympy import ConditionSet, core, diff, Eq, FiniteSet, Intersection, nsolve, Reals, solve, solveset, Symbol

# Variabler
x            = Symbol("x")       # Produksjons-mengden x
x_max        = float()           # Produksjonsmengden som gir størst overskudd
K            = Symbol("K")       # Polynomets navn (vs)
grad         = 2                 # Polynomets grad
poly         = Symbol("poly")    # Polynomet (vs og hs)
x_liste      = [                 # Liste med x-verdier
    10,
    25,
    40,
    70,
    100,
    140,
    180]
y_liste      = [                 # Liste med y-verdier
    270,
    550,
    870,
    1500,
    2200,
    3300,
    4500]
O            = Symbol("O")       # Overskudds-funksjonen
dO           = Symbol("dO")      # Den deriverte av overskudds-funksjonen
rund         = 2                 # -1: Ingen avrunding, 0: Eks: 123.0, n > 0: n siffer etter desimaltegnet, None: Heltall
_blokk       = 1                 # 0: Av, 1: På, Skjul/debug deler av koden med if-blokker

# Definerer funksjoner
def reggis(variabel = Symbol(""),
           grad     = int(),
           x_liste  = list(),
           y_liste  = list(),
           rund     = -1):

    # - Lager polynom-funksjon av valgfri grad vha. regresjon (CAS)

    # Variabler
    _koeff_liste       = list()                # Liste med koeffesienter i fra polyfit()
    _koeff             = float()               # Polynomets koeffisienter
    _ledd              = Symbol("_ledd")       # Polynomets ledd
    _ledd_grad         = int()                 # Polynomets ledd inkludert grad
    _uttrykk           = Symbol("_uttrykk")    # Polynomets uttrykk (hs)

    # polyfit() returnerer en liste med regresjons-koeffesientene
    _koeff_liste = polyfit(x_liste, y_liste, grad)

    # Hver iterasjon lager det neste leddet i polynomet
    for i in range(len(_koeff_liste)):

        # - På 1. iterasjon må _uttrykk nulles ut slik at det ikke blir en
        #   egen variabel i uttrykket, f.eks. "_uttrykk + 0.04*x**2 + 17.05*x + 102.7"
        if i == 0: _uttrykk = 0

        # Runder av koeffesienten
        if rund == -1: _koeff = _koeff_liste[i]
        if rund != -1: _koeff = round(_koeff_liste[i], rund)

        # Lager leddene i fra størst til minst grad
        _ledd_grad = grad - i

        # Formaterer leddene
        if _ledd_grad >= 2: _ledd = _koeff * variabel**_ledd_grad
        if _ledd_grad == 1: _ledd = _koeff * variabel
        if _ledd_grad == 0: _ledd = _koeff

        # Legger det nye leddet til polynomet
        _uttrykk += _ledd

    return _uttrykk

def superløs(variabel = Symbol(""),
             vs       = Symbol(""),
             hs       = Symbol(""),
             likning  = list(),
             rund     = -1,
             debug    = -1):

    vis_datatype = True
    losning_set, losning_set_sub = set(), set()
    losning_liste, losning_element, losning_rund_liste, losning_ut = list(), list(), list(), list()
    losning_rund_status, sett_typ, likn_ant, los_ant, _v_avrund_typ = str(), str(), str(), str(), str()
    losning_rund = float()

    if type(likning) != list: likning_tmp = likning; likning = []; likning.append(likning_tmp)

    if len(likning) == 0:
        likn_ant = "En likning"
        if type(variabel) == type(list()): variabel = variabel[0]
        if type(vs) != core.mul.Mul and type(vs) == type(list()): vs = vs[0]
        if type(hs) != core.mul.Mul and type(hs) == type(list()): hs = hs[0]
        _likning = Eq(vs, hs); losning_set = solveset(_likning, variabel, domain = Reals)

        if len(likning) == 0 and type(losning_set) == FiniteSet:
            sett_typ = "FiniteSet";
            if len(losning_set) == 1: los_ant = "en løsning"
            if len(losning_set) > 1: los_ant = str(len(losning_set)) + " " + "løsninger"

        if len(likning) == 0 and type(losning_set) == ConditionSet:
            sett_typ = "ConditionSet"; losning_element = nsolve(_likning, variabel, 1)
            los_ant = "en numerisk løsning"; losning_set = []; losning_set.append(losning_element)

        if len(likning) == 0 and type(losning_set) == Intersection:
            sett_typ = "Intersection"; losning_set_sub = losning_set.args[1]
            losning_element = losning_set_sub.args; losning_set = [];
            for el in losning_element: losning_set.append(el); rund = -1
            if len(losning_set) == 1: los_ant = "en løsning som er et uttrykk"
            if len(losning_set) > 1: los_ant = str(len(losning_set)) + " " + "løsninger som er uttrykk"

    if len(likning) > 0:
        sett_typ = ""; likn_ant = "Flere likninger (likningssett)"
        losning_set = solve(likning, variabel, dict=True)
        for i in range(len(losning_set)):
            koordinat_verdi_liste = list(); koordinat_variabel_rund_liste = list()
            for koordinat_variabel, koordinat_verdi in losning_set[i].items():
                koordinat_verdi_liste.append(koordinat_verdi)
                losning_rund = round(float(koordinat_verdi), rund)
                koordinat_variabel_rund_liste.append(losning_rund)
            losning_liste.append(koordinat_verdi_liste); losning_rund_liste.append(koordinat_variabel_rund_liste)

    for losning in losning_set:
        if rund == -1:
            _v_avrund_typ = "(eksakt)"; losning_rund_status = "Nei"; losning_rund_liste = ""
            if len(likning) == 0: losning_liste.append(losning); losning_ut = losning_liste
            if len(likning) > 0: losning_ut = losning_liste

        if rund != -1:
            _v_avrund_typ = "(avrundet)"; losning_rund_status = "Ja"
            if len(likning) == 0:
                losning_liste.append(losning); losning_rund = round(losning, rund)
                losning_rund_liste.append(losning_rund); losning_ut = losning_rund_liste
            if len(likning) > 0: losning_ut = losning_rund_liste

    if len(losning_ut) == 1: losning_ut = losning_ut[0]

    if debug == 1:
        print("")
        print(f"*** Debug ****")
        if vis_datatype == True:
            print(f"Data-type                        :: {sett_typ}")
        print(f"Løsnings-type                    :: {likn_ant} med {los_ant} {_v_avrund_typ}")
        print(f"Løsnings-sett                    :: {losning_set}")
        print(f"Løsnings-element(er)             :: {losning_liste}")
        print(f"Avrunding (ja/nei)               :: {losning_rund_status}")
        print(f"Løsnings-elemente(er) avrundet   :: {losning_rund_liste}")
        print(f"-----------------------------------")
        print(f"Løsning ut (returnert) -----------> {losning_ut}")
        print(f"-----------------------------------")

    return losning_ut

# a) Finn O(x)
if _blokk == 1:

    # Lager polynom-funksjon av valgfri grad vha. regresjon (CAS)
    K = reggis(variabel = x,
               grad     = grad,
               x_liste  = x_liste,
               y_liste  = y_liste,
               rund     = rund)

    # Svar-setninger
    svar_a_liste = list()
    svar_a_liste.append(f"")
    svar_a_liste.append(f"Oppg a)")
    svar_a_liste.append(f"")
    svar_a_liste.append(f"- Med regresjon fra numpy (polyfit) ser vi at hvis bedriften")
    svar_a_liste.append(f"  produserer x enheter, så vil")
    svar_a_liste.append(f"")
    svar_a_liste.append(f"    {K}")
    svar_a_liste.append(f"")
    svar_a_liste.append(f"  være en god model for det måntlige overskuffet (i tusen kroner)")

    # Print svar-setninger
    for svar in svar_a_liste: print(svar)

# b) Størst overskudd
if _blokk == 1:

    # Definerer uttrykket for overskudds-funksjonen, O
    O = -0.041*x**2 + 11*x - 103

    # Deriverer O mhp. x og får dO = 11 - 0.082*x
    dO = diff(O, x) # O'(x)

    # Løser likningen dO = 0 
    x_max = superløs(variabel = x,
                     vs       = dO,
                     hs       = 0,
                     rund     = None,
                     debug    = -1)

    # Svar-setninger
    svar_b_liste = list()
    svar_b_liste.append(f"")
    svar_b_liste.append(f"Oppg b)")
    svar_b_liste.append(f"")
    svar_b_liste.append(f"- Negativ a-koeffisient gir et toppunkt på O")
    svar_b_liste.append(f"- Produksjonsmengden x = {x_max} gir størst overskudd")

    # Print svar-setninger
    for svar in svar_b_liste: print(svar)



# 🚀 programmering.no | 🤓 matematikk.as
# - Se om intervallet har armstrong-tall
# - Def: I et armstrong-tall er summen av siffrene
#   opphøyd i antall siffer lik tallet
# - Eks: 371 => 3 siffer
#   3^(3) + 7^(3) + 1^(3) = 27 + 343 + 1 = 371

# Verdier
min = 368
max = 373

# Funksjon for å se om intervallet har armstrong-tall
def er_armstrong_tall_intervall(min, max):

    # Itererer gjennom intervallet
    for n in range(int(min), int(max) + 1):

        # Variabler
        n_kopi          = n             # Kopier startverdien til n
        siffer_ant      = len(str(n))   # Eks: str(371) -> "371" => len("371") = 3 (siffer)
        sum             = 0             # Nullstill

        # Itererer gjennom sifrene i n (stop når n = 0)
        while n != 0:

            # Få det bakerste sifferet med modulus 10 (resten av å dele på 10)
            siffer = n % 10 # Eks: 371 % 10 = 1 fordi 371 / 10 = 370 (heltall) + 1 (rest)

            # Oppdater sum for hver iterasjon
            sum = sum + (siffer**siffer_ant)

            # n / 10 => Få det neste bakerste sifferet
            n = int(n / 10) # Eks: 371 (3 siffer) / 10 = 37.1 => int(37.1) = 37 (2 siffer) => ... => int(3.7) = 3 (1 siffer)

        # Se om sum er lik opprinnelig verdi til n
        if sum == n_kopi: print(f"Er {n_kopi} et armstrong-tall? Ja")
        else:             print(f"Er {n_kopi} et armstrong-tall? Nei")

# Armstrong-tall eller ikke
er_armstrong_tall_intervall(min, max)



# 🚀 programmering.no | 🤓 matematikk.as
# Sinus 1T (2020) - BrettBoka - 1.5 Formler (s. 28) 
# Eksempel 1.5 a) og b) Vindpark I - Input (Matematikk AS)

# Blokk med definisjoner
_blokk             = 1             # 0: Av, 1: På, Skjul/debug deler av koden med if-blokker
if _blokk == 1:

    # Konstanter
    pi                 = 3.14          # Pi avrundet til 3.14
    turbin_radius      = 58.5          # Rotorbladets lengde, [m]
    turbin_areal       = 0             # Turbinens areal, [m^2], A = pi * r^2
    turbin_k           = 0.0003        # Turbin-konstant (lufttetthet og virkningsgrad), [W(s^3/m^5)]
    turbin_effekt      = 0             # Turbinens effekt (sekund), [kW]
    vindstyrke         = 5             # Lett bris, [m/s]

    # Enheter
    enhet_m_kvad       = "m^2"         # Kvadratmeter
    enhet_kw           = "kW"          # Kilowatt

    # Tekst
    txt_oppg           = "Eks 1.5 a) og b)"
    txt_strek          = "----------------"
    txt_a              = "a) Turbinens areal           :"
    txt_b              = "b) Turbinens effekt (sekund) :"

    # Innstillinger
    inp                = 1             # # 0: Boken, 1: Bruker
    avrund_valg        = 1             # # 0: Ikke avrunding, 1: Avrunding
    avrund_ant         = None          # # None: 12.78 -> 13, 0: 12.78 -> 13.0, 1: 12.78 -> 12.8, 2: 12.78 -> 12.78

# Input
print("")
print(txt_strek)
print(txt_oppg)
print(txt_strek)
if inp == 1:

    print("")
    turbin_radius = input("Hvor lange er rotorbladene [m]? ")
    vindstyrke = input("Hva er vindstyrken [m/s]? ")

    # Type casting: str -> float
    turbin_radius       = float(turbin_radius)
    vindstyrke          = float(vindstyrke)

# a) Turbinens areal
turbin_areal       = pi * (turbin_radius**2)

# b) Turbinens effekt (sekund)
turbin_effekt      = turbin_k * (vindstyrke**3) * turbin_areal

# Avrunding
if avrund_valg == 1:
    turbin_areal   = round(turbin_areal, avrund_ant)
    turbin_effekt  = round(turbin_effekt, avrund_ant)

# Print
print("")
print(txt_a, turbin_areal, enhet_m_kvad)
print(txt_b, turbin_effekt, enhet_kw)
print("")



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Vår (Matematikk AS)
# Oppgave 4 a) Aritmetisk- og geometrisk rekke - Ukentlig beløp de 4 første ukene 

lonn_0      = 100       # Ukelønn første uke
a_n         = lonn_0    # Tilbud 1: Ukelønn første uke
a_d         = 10        # Tilbud 1: Aritmetisk økning (differanse)
g_n         = lonn_0    # Tilbud 2: Ukelønn første uke
g_k         = 1.05      # Tilbud 2: Geometrisk økning (kvotient)
uker        = 4         # Antall uker
desimal     = 2         # Antall desimaler i avrunding

# Print-funksjon
def printing(i, belop): print(f"Tilbud 1 | Uke {i} | Beløp: {round(belop, desimal)}")

# Oppgave a)
print("Oppg a)")

# Aritmetisk rekke
for i in range(1, uker + 1): printing(i, a_n); a_n += a_d

# Geometrisk rekke 
print("")
for i in range(1, uker + 1): printing(i, g_n); g_n *= g_k



import matematikk as mt

def farge_txt(txt = str(), tab_farge_typ = str()):

    # Farge
    if tab_farge_typ == "blå":          return f"{mt.Fore.BLUE}      {txt}{mt.Style.RESET_ALL}"
    if tab_farge_typ == "grønn":        return f"{mt.Fore.GREEN}     {txt}{mt.Style.RESET_ALL}"
    if tab_farge_typ == "magentea":     return f"{mt.Fore.MAGENTA}   {txt}{mt.Style.RESET_ALL}"
    if tab_farge_typ == "rød":          return f"{mt.Fore.RED}       {txt}{mt.Style.RESET_ALL}"
    if tab_farge_typ == "gul":          return f"{mt.Fore.YELLOW}    {txt}{mt.Style.RESET_ALL}"

    # Ingen farge (bypass)
    return txt


##########################################
# farge_txt()
##########################################

# Alias > 1 > farge_txt
farge_tekst = farge_txt

# Alias > 2 > ...

